#!/usr/bin/env python
import glob
import os
import optparse
import shutil
import socket
import subprocess
import time
import xmlrpclib


#----------------------------------------------------------
# Utils
#----------------------------------------------------------
join = os.path.join

def mkdir(d):
    if not os.path.isdir(d):
        os.makedirs(d)

def url2dir(n):
    return n.replace('/','_').replace(':','').replace('~','')

def system(l,chdir=None):
    print l
    if chdir:
        cwd = os.getcwd()
        os.chdir(chdir)
    if isinstance(l,list):
        rc=os.spawnvp(os.P_WAIT, l[0], l)
    elif isinstance(l,str):
        tmp=['sh','-c',l]
        rc=os.spawnvp(os.P_WAIT, tmp[0], tmp)
    if chdir:
        os.chdir(cwd)
    return rc

#----------------------------------------------------------
# Stages
#----------------------------------------------------------

def init(o):
    for i in [o.repo, o.work, o.pkg, o.pub, join(o.pub,"src"), join(o.pub,"deb"), join(o.pub,"exe"), join(o.pub,"rpm")]:
        mkdir(i)
    if not os.path.isdir(join(o.repo,'.bzr')):
        system(['bzr','init-repo',o.repo])

def branch_or_update(b,d):
    if os.path.isdir(d):
        system(['bzr','pull','-d',d,'--overwrite'])
    else:
        system(['bzr','branch',b,d])

def update(o):
    branch_or_update(o.addons_branch,o.addons_dir)
    branch_or_update(o.server_branch,o.server_dir)
    branch_or_update(o.web_branch,o.web_dir)

def rsync(o):
    system(["rsync","-a","--exclude",".bzr","--delete", "%s/"%o.server_dir, o.work])
    system(["rsync","-a","--exclude",".bzr", "%s/"%o.addons_dir, o.work_addons])
    system(["rsync","-a","--exclude",".bzr", "%s/addons/"%o.web_dir, o.work_addons])

def version(o):
    open(join(o.work,'openerp','release.py'),'a').write('version = "%s-%s"'%(o.version,o.timestamp))

def sdist(o):
    cmd=['python','setup.py', '--quiet', 'sdist', '-d','../pkg']
    system(cmd,o.work)

def bdist_rpm(o):
    cmd=['python','setup.py', '--quiet', 'bdist_rpm', '-d','../pkg']
    system(cmd,o.work)
    shutil.rmtree(join(o.work,'dist'))

def debian(o):
    cmd=['sed','-i','1s/^.*$/openerp (%s-%s-1) testing; urgency=low/'%(o.version,o.timestamp),'debian/changelog']
    system(cmd,o.work)
    cmd=['dpkg-buildpackage','-rfakeroot']
    system(cmd,o.work)
    l = glob.glob(join(o.build,'openerp_*'))
    for i in l:
        shutil.move(i,o.pkg)

class KVM(object):
    def __init__(self, o, image, ssh_key=''):
        self.o = o
        self.image = image
        self.ssh_key = ssh_key
        self.login = 'openerp'
    def start(self):
        l="kvm -net nic -net user,hostfwd=tcp:127.0.0.1:10022-:22,hostfwd=tcp:127.0.0.1:18069-:8069,hostfwd=tcp:127.0.0.1:15432-:5432 -drive".split(" ")
        #l.append('file=%s,if=virtio,index=0,boot=on,snapshot=on'%self.image)
        l.append('file=%s,snapshot=on'%self.image)
        #l.extend(['-vnc','127.0.0.1:1'])
        l.append('-nographic')
        print l
        pid=os.spawnvp(os.P_NOWAIT, l[0], l)
        time.sleep(5)
        try:
            self.run()
        finally:
            os.kill(pid,15)
            time.sleep(5)
    def ssh(self,cmd):
        l=['ssh','-o','UserKnownHostsFile=/dev/null','-o','StrictHostKeyChecking=no','-p','10022','-i',self.ssh_key,'%s@127.0.0.1'%self.login,cmd]
        system(l)
    def rsync_ssh(self):
        return 'ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p 10022 -i %s'%self.ssh_key
    def rsync(self,src,dest):
        l=['rsync','-rtv','-e',self.rsync_ssh(),'--delete',src,'%s@127.0.0.1:%s'%(self.login,dest)]
        system(l)
    def run(self):
        pass

class KVMDebianTestTgz(KVM):
    def run(self):
        l = glob.glob(join(self.o.pkg,'*%s.tar.gz'%self.o.timestamp))
        self.rsync(l[0],'src/')
        script = """
            tar xzvf src/*.tar.gz
            cd openerp*
            sudo python setup.py install
            sudo su - postgres -c "createuser -s $USER"
            createdb t1
            openerp-server --stop-after-init -d t1 -i ` python -c "import os;print ','.join([i for i in os.listdir('openerp/addons') if i not in ['auth_openid','caldav','document_ftp']]),;" `
        """
        self.ssh(script)
        self.ssh('nohup openerp-server >/dev/null 2>&1 &')
        time.sleep(2)
        l = xmlrpclib.ServerProxy('http://127.0.0.1:18069/xmlrpc/object').execute('t1',1,'admin','ir.module.module','search',[('state','=','installed')])
        i = len(l)
        if i >= 190:
            print "Tgz install: ",i," module installed"
        else:
            raise Exception("Tgz install failed only %s installed"%i)
        time.sleep(2)

class KVMDebianTestDeb(KVM):
    def run(self):
        l = glob.glob(join(self.o.pkg,'*%s*.deb'%self.o.timestamp))
        self.rsync(l[0],'deb/')
        script = """
            sudo dpkg -i deb/*
            sudo su - postgres -c "createuser -s $USER"
            createdb t1
            openerp-server --stop-after-init -d t1 -i base
        """
        #` python -c "import os;print ','.join([i for i in os.listdir('/usr/share/pyshared/openerp/addons') if i not in ['auth_openid','caldav','document_ftp']]),;" `
        self.ssh(script)
        time.sleep(2)
        l = xmlrpclib.ServerProxy('http://127.0.0.1:18069/xmlrpc/object').execute('t1',1,'admin','ir.module.module','search',[('state','=','installed')])
        i = len(l)
        if i >= 1:
            print "Deb install: ",i," module installed"
        else:
            raise Exception("Tgz install failed only %s installed"%i)
        time.sleep(2)

class KVMWinBuildExe(KVM):
    def run(self):
        # The revno is the one from the server
        #revno = subprocess.Popen(['bzr', 'revno', self.o.server_dir], stdout=subprocess.PIPE).communicate()[0].strip()
        self.login = 'Naresh'
        self.ssh("mkdir -p openerp-packaging/server")
        self.ssh("mkdir -p openerp-packaging/windows")
        system('rsync -av -e "%s" --delete --exclude .bzr/ --exclude .bzrignore --exclude /packages/ %s/ Naresh@127.0.0.1:openerp-packaging/server' % (self.rsync_ssh(), self.o.work))
        f = open('windows/Makefile.version','w')
        f.write("VERSION=%s-%s\n" % (self.o.version,self.o.timestamp))
        f.close()
        system('rsync -av -e "%s" --delete windows/ Naresh@127.0.0.1:openerp-packaging/windows/'%(self.rsync_ssh(),))
        self.ssh("ls openerp-packaging")
        self.ssh("cd openerp-packaging/windows;time make allinone;")
        system('rsync -av -e "%s" Naresh@127.0.0.1:openerp-packaging/windows/files/ %s/%s/ '%(self.rsync_ssh(), self.o.pkg, self.o.version))
        print "KVMWinBuildExe.run(): done"

class KVMWinTestExe(KVM):
    def run(self):
        setuppath = "%s/%s/openerp-allinone-setup-%s-%s.exe" % (self.o.pkg, self.o.version, self.o.version, self.o.timestamp)
        print setuppath
        setupdir, setupexe = os.path.split(setuppath)
        setup_no_ext, ext = os.path.splitext(setupexe)
        system('rsync -av -e "%s" --delete %s Naresh@127.0.0.1:'%(self.rsync_ssh(), setuppath))
        self.login = 'Naresh'
        #system('ssh -i %s -p 10022 Naresh@127.0.0.1 "./%s /S"'%(self.ssh_key, setupexe))
        self.ssh("./%s /S" % setupexe)
        # Change postgres config so it accepts our connections.
        system('rsync -av -e "%s" --delete windows/test/pg_hba.conf "Naresh@127.0.0.1:/cygdrive/c/Program\ Files/OpenERP\ %s-%s/PostgreSQL/data/pg_hba.conf"'%(self.rsync_ssh(), self.o.version, self.o.timestamp))
        system('rsync -av -e "%s" --delete windows/test/postgresql.conf "Naresh@127.0.0.1:/cygdrive/c/Program\ Files/OpenERP\ %s-%s/PostgreSQL/data/postgresql.conf"'%(self.rsync_ssh(), self.o.version, self.o.timestamp))
        port = '10022'
        ip = 'Naresh@127.0.0.1'
        #system("""ssh -i %s -p %s %s 'net stop "PostgreSQL For OpenERP"'"""%(self.ssh_key, port, ip))
        #system("""ssh -i %s -p %s %s 'net start "PostgreSQL For OpenERP"'"""%(self.ssh_key, port, ip))
        self.ssh('net stop "PostgreSQL For OpenERP"')
        self.ssh('net start "PostgreSQL For OpenERP"')
        subprocess.call('createdb -e -h 127.0.0.1 -p 15432 -U openpg pack'.split(' '), env={'PGPASSWORD':'openpgpwd'})
        #system('ssh -i %s -p %s %s "/cygdrive/c/Program\ Files/OpenERP\ %s-%s/server/openerp-server.exe -d pack -i base,report_webkit --stop-after-init --log-level=test"'%(self.ssh_key, port, ip, self.o.version, self.o.timestamp))
        self.ssh("/cygdrive/c/Program\ Files/OpenERP\ %s-%s/server/openerp-server.exe -d pack -i base,report_webkit --stop-after-init --log-level=test"%(self.o.version, self.o.timestamp))
        system("scp -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -i %s -P %s '%s:/cygdrive/c/Program\ Files/OpenERP\ %s-%s/server/openerp-server.log' %s/%s.log"%(self.ssh_key, port, ip, self.o.version, self.o.timestamp, 'results', setup_no_ext))

class KVMWinBuildGtk(KVM):
    def run(self):
        pass

def publish(o):
    l = glob.glob(join(o.pkg,'*%s-1*.rpm'%o.timestamp.replace('-','_')))
    for i in l:
        shutil.move(i,join(o.pub,'rpm'))
    l = glob.glob(join(o.pkg,'*_*%s-1*'%o.timestamp))
    for i in l:
        shutil.move(i,join(o.pub,'deb'))
    system('dpkg-scanpackages . /dev/null | gzip -9c > Packages.gz',join(o.pub,'deb'))
    l = glob.glob(join(o.pkg,'*%s*.tar.gz'%o.timestamp))
    for i in l:
        shutil.move(i,join(o.pub,'src'))
    #shutil.rmtree(o.work)

#----------------------------------------------------------
# Options and Main
#----------------------------------------------------------

def options():
    op = optparse.OptionParser()
    op.add_option("-b", "--build", default='.', help="build directory (%default)", metavar="DIR")
    op.add_option("-v", "--version", default='6.1dev', help="version (%default)")
    op.add_option("-a", "--addons-branch", default='lp:~openerp/openobject-addons/trunk', help="%default")
    op.add_option("-s", "--server-branch", default='lp:~openerp/openobject-server/trunk', help="%default")
    op.add_option("-w", "--web-branch", default='lp:~openerp/openerp-web/trunk', help="%default")
    op.add_option("-c", "--client-branch", default='lp:~openerp/openobject-client/trunk', help="%default")
    op.add_option("", "--vm-debian-image", default='/home/odoo/vm/debian6/debian6.vmdk', help="%default")
    op.add_option("", "--vm-debian-ssh-key", default='/home/odoo/vm/debian6/debian6_id_rsa', help="%default")
    op.add_option("", "--vm-winxp-image", default='/home/odoo/vm/winxp26/winxp26.vdi', help="%default")
    op.add_option("", "--vm-winxp-ssh-key", default='/home/odoo/vm/winxp26/id_rsa', help="%default")
    (o, args) = op.parse_args()
    # derive other options
    o.timestamp = time.strftime("%Y%m%d-%H%M%S",time.gmtime())
    o.repo = join(o.build, 'repo')
    o.pkg = join(o.build, 'pkg')
    o.pub = join(o.build, 'pub')
    o.work = join(o.build, 'openerp-%s-%s'%(o.version,o.timestamp))
    o.work_addons = join(o.work, 'openerp', 'addons')
    o.addons_dir = join(o.repo, url2dir(o.addons_branch))
    o.server_dir = join(o.repo, url2dir(o.server_branch))
    o.web_dir = join(o.repo, url2dir(o.web_branch))
    return o

def main():
    o = options()
    print "-- init"
    init(o)
    print "-- update"
    update(o)
    print "-- rsync"
    rsync(o)
    print "-- version"
    version(o)
    #print "-- sdist"
    #sdist(o)
    #print "-- bdist_rpm"
    #bdist_rpm(o)
    #print "-- debian"
    #debian(o)
    if os.path.isfile(o.vm_debian_image):
        #print "-- debian test tgz"
        #KVMDebianTestTgz(o, o.vm_debian_image, o.vm_debian_ssh_key).start()
        #print "-- debian test deb"
        #KVMDebianTestDeb(o, o.vm_debian_image, o.vm_debian_ssh_key).start()
        pass
    if os.path.isfile(o.vm_winxp_image):
        print "-- windows build gtk"
        KVMWinBuildExe(o, o.vm_winxp_image, o.vm_winxp_ssh_key).start()
        print "-- windows build gtk"
        KVMWinTestExe(o, o.vm_winxp_image, o.vm_winxp_ssh_key).start()
        #print "-- windows build gtk"
        #KVMWinBuildGtk(o, o.vm_winxp_image, o.vm_winxp_ssh_key).start()
    #print "-- publish"
    #publish(o)

if __name__ == '__main__':
    main()
